<?php
/*
Plugin Name: Get Feed
Description: Adds a custom page under Tools menu to fetch and display an feed.
Version: 1.0
Author: Nafiul Islam
Author URI: https://version001.com/
*/

// Register a function to create the Tools submenu page
function get_feed_page() {
    add_submenu_page(
        'edit.php',            // Parent menu slug
        'Get Feed Page',        // Page title
        'Get Feed',             // Menu title
        'manage_options',       // Required capability
        'get-feed',             // Menu slug
        'get_feed_callback'     // Callback function
    );
}

add_action('admin_menu', 'get_feed_page');

// Callback function to display content on the Get Feed page
function get_feed_callback() {
    $feed_url_value = isset($_POST['feed_url']) ? sanitize_text_field($_POST['feed_url']) : '';

    $message = '';

    if (isset($_POST['fetch_feed'])) {
        $feed_url = sanitize_text_field($_POST['feed_url']);
        $feed_content = file_get_contents($feed_url);
        $file_content_with_br = nl2br($feed_content);
        $channel_blocks = explode('<channel>', $file_content_with_br);
        array_shift($channel_blocks);
        $channel_data = array();

        // Loop through each channel block
        foreach ($channel_blocks as $channel_block) {
            // Find the closing </channel> tag
            $end_pos = strpos($channel_block, '</channel>');
            if ($end_pos !== false) {
                // Extract the content between <channel> and </channel>
                $channel_content = substr($channel_block, 0, $end_pos);

                // Store the extracted content in the array
                $channel_data[] = $channel_content;
            }
        }

        if (!empty($channel_data)) {
            foreach ($channel_data as $v_data) {
                preg_match('/<title>(.*?)<\/title>/s', $v_data, $title_matches);
                preg_match('/<description>(.*?)<\/description>/s', $v_data, $description_matches);
                preg_match('/<pubDate>(.*?)<\/pubDate>/s', $v_data, $pubdate_matches);
                preg_match('/<item>(.*?)<\/item>/s', $v_data, $content_matches);

                if (isset($title_matches[1])) {
                    $title = strip_tags(htmlspecialchars_decode($title_matches[1]));
                }

                if (!post_exists($title)) {
                    // Create a new draft post
                    $pubdate = isset($pubdate_matches[1]) ? $pubdate_matches[1] : '';

                    // Convert custom date to timestamp
                    $timestamp = strtotime($pubdate);
                    // Convert timestamp to WordPress date format
                    $wp_date_format = date_i18n('Y-m-d H:i:s', $timestamp);

                    $post_excerpt = isset($description_matches[1]) ? $description_matches[1] : '';
                    $post_content = isset($content_matches[1]) ? $content_matches[1] : '';
                    $post_data = array(
                        'post_title' => $title,
                        'post_content' => $post_content,
                        'post_excerpt' => $post_excerpt,
                        'post_date' => $wp_date_format,
                        'post_date_gmt' => $wp_date_format,
                        'post_status' => 'draft',
                    );

                    $post_id = wp_insert_post($post_data);

                    if (!is_wp_error($post_id)) {
                        // Set the post permalink
                        add_post_meta($post_id, '_custom_permalink', true);
                        $message .= 'Draft post created: ' . $title . '<br>';
                    }
                } else {
                    $message .= 'Post already exists: ' . $title . '<br>';
                }
            }
        }
    }

    ?>
    <div class="wrap">
        <h2>Get Feed and Create Draft Posts</h2>

        <form method="post">
            <label for="feed_url">Enter feed URL:</label>
            <input type="text" name="feed_url" required id="feed_url" value="<?php echo esc_attr($feed_url_value); ?>">
            <input type="submit" name="fetch_feed" value="Click To Import + Wait">
        </form>

        <?php if (!empty($message)) : ?>
            <div class="updated">
                <p><?php echo $message; ?></p>
            </div>
        <?php endif; ?>
    </div>
    <?php
}

?>
